require "members_report/gruff/line"

class MembersReportController < ApplicationController

  before_filter :find_project
  before_filter :authorize

  menu_item :overview

  helper :timelog

  def me
    @members = [User.current]
    @selected_members = [User.current.id]
    generate_data
  end

  def all
    @members = @project.members.collect{|member| member.user}
    @selected_members = ((params[:selected_members].is_a?(String) ? eval(params[:selected_members]) : params[:selected_members]) || @members.collect{|m| m.id}).collect{|m| m.to_i}
    generate_data
  end

  def graph_lines
    g = MembersReport::Gruff::Line.new(params[:s] || "800x400")
    g.hide_title = true
    g.theme = graph_theme
    g.labels_x_rotation = -15.0
    g.marker_count = 5
    g.marker_font_size = 12
    g.legend_font_size = 12
    g.legend_box_size = 12
    g.line_width = 2
    g.dot_radius = 0

    labels = params[:l] || []
    titles = params[:t] || []
    values = params[:v] || []

    i = 0
    while i < titles.size
      g.labels[i] = titles[i]
      i += 1
    end

    i = 0
    while i < labels.size and (i * titles.size) < values.size
      g.data(labels[i], values[(i * titles.size), titles.size].collect{|v| v.to_f})
      i += 1
    end

    headers["Content-Type"] = "image/png"
    send_data(g.to_blob, :type => "image/png", :disposition => "inline")
  end

private

  def find_project
    @project = Project.find(params[:project_id])
  rescue ActiveRecord::RecordNotFound
    render_404
  end

  DEFAULT_PEDIODS = 4

  def retrieve_date_range
    @free_period = false
    @from, @to = nil, nil
    @group_by = (params[:group_by] || :week).to_sym

    if params[:period_type] == '1' || (params[:period_type].nil? && !params[:period].nil?)
      case params[:period].to_s
      when 'today'
        @from = @to = Date.today
      when 'yesterday'
        @from = @to = Date.today - 1
      when 'current_week'
        @from = Date.today - (Date.today.cwday - 1)%7
        @to = @from + 6
      when 'last_week'
        @from = Date.today - 7 - (Date.today.cwday - 1)%7
        @to = @from + 6
      when '7_days'
        @from = Date.today - 7
        @to = Date.today
      when 'current_month'
        @from = Date.civil(Date.today.year, Date.today.month, 1)
        @to = (@from >> 1) - 1
      when 'last_month'
        @from = Date.civil(Date.today.year, Date.today.month, 1) << 1
        @to = (@from >> 1) - 1
      when '30_days'
        @from = Date.today - 30
        @to = Date.today
      when 'current_year'
        @from = Date.civil(Date.today.year, 1, 1)
        @to = Date.civil(Date.today.year, 12, 31)
      end
    elsif params[:period_type] == '2' || (params[:period_type].nil? && (!params[:from].nil? || !params[:to].nil?))
      begin; @from = params[:from].to_s.to_date unless params[:from].blank?; rescue; end
      begin; @to = params[:to].to_s.to_date unless params[:to].blank?; rescue; end
      @free_period = true
    else
      # default
    end

    @from, @to = @to, @from if @from && @to && @from > @to

    @to   ||= (TimeEntry.maximum(:spent_on, :include => :project, :conditions => Project.allowed_to_condition(User.current, :view_time_entries)) || Date.today)
    case @group_by
    when :day
    when :week
      @to = @to + 6 - ((@to.cwday - 1) % 7)
    when :month
      @to = (Date.civil(@to.year, @to.month, 1) >> 1) - 1
    end

    if @from.nil?
      case @group_by
      when :day
        @from = @to - DEFAULT_PEDIODS + 1
      when :week
        @from = @to - (DEFAULT_PEDIODS * 7) + 1
      when :month
        @from = (@to << DEFAULT_PEDIODS) + 1
      end
      @free_period = true
    end

    case @group_by
    when :day
    when :week
      @from = @from - ((@from.cwday - 1) % 7)
    when :month
      @from = Date.civil(@from.year, @from.month, 1)
    end
  end

  def generate_empty_periods(from, to, group_by)
    empty_periods = []
    date = from
    while date <= to
      from_date = date
      case group_by
      when :day
        to_date = date
        date += 1
      when :week
        to_date = date + 6
        date += 7
      when :month
        to_date = (date >> 1) - 1
        date >>= 1
      end
      empty_periods << {:from => from_date, :to => to_date, :time_entries => [], :total => 0.0, :speed_total => 0.0, :media => 0.0, :speed_media => 0.0, :count => 0, :bugs => []}
    end
    empty_periods
  end

  def generate_data
    retrieve_date_range

    @empty_periods = generate_empty_periods(@from, @to, @group_by)
    @users_time_entries = {:values => {},
                           :total => {:periods => generate_empty_periods(@from, @to, @group_by), :total => 0.0, :speed_total => 0.0, :media => 0.0, :speed_media => 0.0, :count => 0, :bugs => []}}
    @members.each do |user|
      if @selected_members.include?(user.id)
        @users_time_entries[:values][user.id] = {:user => user, :periods => generate_empty_periods(@from, @to, @group_by), :total => 0.0, :speed_total => 0.0, :media => 0.0, :speed_media => 0.0, :count => 0, :bugs => []}
        i = 0
        @users_time_entries[:values][user.id][:periods].each do |period|
          period[:time_entries] = TimeEntry.find(:all,
                                                 :conditions => {:project_id => @project.id,
                                                                 :user_id => user.id,
                                                                 :spent_on => period[:from]..period[:to]})
          period[:total] = 0.0
          period[:speed_total] = 0.0
          period[:bugs] = []
          period[:time_entries].each do |time_entry|
            issue = time_entry.issue
            if !issue.nil? and issue.respond_to?("speed")
              speed = issue.speed
            else
              speed = 1.0
            end
            if !issue.nil? and issue.respond_to?("bugs")
              bugs = issue.bugs
            else
              bugs = []
            end
            period[:total] += time_entry.hours
            period[:speed_total] += time_entry.hours * speed
            period[:bugs] |= bugs
          end
          period[:time_on_bugs] = period[:bugs].collect{|bug| bug.total_hours}.reduce(:+) || 0.0
          period[:media] = period[:time_entries].size == 0 ? 0.0 : period[:total] / period[:time_entries].size
          period[:speed_media] = period[:time_entries].size == 0 ? 0.0 : period[:speed_total] / period[:time_entries].size
          @users_time_entries[:values][user.id][:total] += period[:total]
          @users_time_entries[:values][user.id][:speed_total] += period[:speed_total]
          @users_time_entries[:values][user.id][:count] += 1 if period[:total] > 0.0
          @users_time_entries[:values][user.id][:bugs] |= period[:bugs]
          @users_time_entries[:total][:periods][i][:total] += period[:total]
          @users_time_entries[:total][:periods][i][:speed_total] += period[:speed_total]
          @users_time_entries[:total][:periods][i][:count] += 1 if period[:total] > 0.0
          @users_time_entries[:total][:periods][i][:bugs] |= period[:bugs]
          i += 1
        end
        @users_time_entries[:values][user.id][:media] = @users_time_entries[:values][user.id][:count] == 0 ? 0.0 : @users_time_entries[:values][user.id][:total] / @users_time_entries[:values][user.id][:count]
        @users_time_entries[:values][user.id][:speed_media] = @users_time_entries[:values][user.id][:count] == 0 ? 0.0 : @users_time_entries[:values][user.id][:speed_total] / @users_time_entries[:values][user.id][:count]
        @users_time_entries[:total][:total] += @users_time_entries[:values][user.id][:total]
        @users_time_entries[:total][:speed_total] += @users_time_entries[:values][user.id][:speed_total]
        @users_time_entries[:total][:count] += 1 if @users_time_entries[:values][user.id][:total] > 0.0
        @users_time_entries[:total][:bugs] |= @users_time_entries[:values][user.id][:bugs]
      end
    end
    i = 0
    while i < @empty_periods.size
      @users_time_entries[:total][:periods][i][:media] = @users_time_entries[:total][:periods][i][:count] == 0 ? 0.0 : @users_time_entries[:total][:periods][i][:total] / @users_time_entries[:total][:periods][i][:count]
      @users_time_entries[:total][:periods][i][:speed_media] = @users_time_entries[:total][:periods][i][:count] == 0 ? 0.0 : @users_time_entries[:total][:periods][i][:speed_total] / @users_time_entries[:total][:periods][i][:count]
      i += 1
    end
    @users_time_entries[:total][:media] = @users_time_entries[:total][:count] == 0 ? 0.0 : @users_time_entries[:total][:total] / @users_time_entries[:total][:count]
    @users_time_entries[:total][:speed_media] = @users_time_entries[:total][:count] == 0 ? 0.0 : @users_time_entries[:total][:speed_total] / @users_time_entries[:total][:count]
  end

  def graph_theme
    {:colors => ["#DB2626", "#6A6ADB", "#64D564", "#F727F7", "#EBEB20",
                 "#303030", "#12ABAD", "#808080", "#B7580B", "#316211",
                 "#F0A556", "#835288", "#88B4DA", "#9EE8D6", "#A7CD70"],
     :marker_color => "#AAAAAA",
     :background_colors => ["#FFFFFF", "#FFFFFF"]}
  end

end
